import { Text } from "../../doc/src";
export interface Mapping {
    mapPos(pos: number, bias?: number, trackDel?: boolean): number;
}
export declare class ChangeDesc implements Mapping {
    readonly from: number;
    readonly to: number;
    readonly length: number;
    constructor(from: number, to: number, length: number);
    readonly invertedDesc: ChangeDesc;
    mapPos(pos: number, bias?: number, trackDel?: boolean): number;
    toJSON(): any;
    static fromJSON(json: any): ChangeDesc;
}
export declare class Change extends ChangeDesc {
    readonly from: number;
    readonly to: number;
    readonly text: ReadonlyArray<string>;
    constructor(from: number, to: number, text: ReadonlyArray<string>);
    invert(doc: Text): Change;
    apply(doc: Text): Text;
    map(mapping: Mapping): Change | null;
    readonly desc: ChangeDesc;
    toJSON(): any;
    static fromJSON(json: any): Change;
}
export declare class ChangeSet<C extends ChangeDesc = Change> implements Mapping {
    readonly changes: ReadonlyArray<C>;
    readonly mirror: ReadonlyArray<number>;
    constructor(changes: ReadonlyArray<C>, mirror?: ReadonlyArray<number>);
    readonly length: number;
    getMirror(n: number): number | null;
    append(change: C, mirror?: number): ChangeSet<C>;
    appendSet(changes: ChangeSet<C>): ChangeSet<C>;
    static empty: ChangeSet<any>;
    mapPos(pos: number, bias?: number, trackDel?: boolean): number;
    /** @internal */
    mapInner(pos: number, bias: number, trackDel: boolean, fromI: number, toI: number): number;
    partialMapping(from: number, to?: number): Mapping;
    changedRanges(): ChangedRange[];
    readonly desc: ChangeSet<ChangeDesc>;
    toJSON(): any;
    static fromJSON<C extends ChangeDesc>(ChangeType: {
        fromJSON: (json: any) => C;
    }, json: any): ChangeSet<C>;
}
export declare class ChangedRange {
    readonly fromA: number;
    readonly toA: number;
    readonly fromB: number;
    readonly toB: number;
    constructor(fromA: number, toA: number, fromB: number, toB: number);
    join(other: ChangedRange): ChangedRange;
    addToSet(set: ChangedRange[]): void;
}
